#ifndef INTERFACE_H
#define INTERFACE_H

#include <QTimer>
#include <QImage>
#include "../common/sources/system/cmaths.h"
#include "character.h"
#include "mouse.h"
#include <map>
#include <string>

#define MAIN_INTERFACE_COLOR    QColor(0, 204, 204)

#define CHAMP_PORTRAIT_WIDTH    32
#define CHAMP_PORTRAIT_HEIGHT   29
#define ITEM_WIDTH    16
#define ITEM_HEIGHT   16

class CInterface
{
public:
    // Main state of the interface
    enum EMainState
    {
        eMainGame = 0,  // standard window with the 3D view
        eMainInventory  // Inventory sheet
    };

    // States of the weapon's area
    enum EWeaponsAreaState
    {
        eWeaponsAreaWeapons = 0,    // The 4 weapons boxes
        eWeaponsAreaAttacks,        // The attacks of a weapon
        eWeaponsAreaDamage          // The damages when we hit a monster
    };

    // fonts
    enum EFonts
    {
        eFontStandard = 0,
        eFontScroll,
        eFontWalls
    };

    CInterface();
    ~CInterface();

    void    display(QImage* image);
    void    update(SMouseArea *clickedArea);
    bool    isInInventory();
    CRect   getChampionPortraitRect(int num);
    CRect   getItemRect(int num);
    void    drawText(QImage* image, CVec2 pos, EFonts font, const char* text, QColor color);
    void    readTextDB();
    std::string getText(std::string id);
    void    enterResurrect(int num);

private:
    bool    isChampionEmpty(int num);
    bool    isChampionDead(int num);

    bool    isWeaponEmpty(int num);
    bool    isWeaponGreyed(int num);

    bool    isArrowsGreyed();
    bool    isSpellsGreyed();

    void    drawBar(QImage* image, int num, CVec2 pos, CCharacter::EStatsNames statName);
    void    drawChampion(QImage* image, int num);
    void    drawSpells(QImage* image);
    void    drawWeaponsArea(QImage* image);
    void    drawWeapon(QImage* image, int num);
    void    drawArrows(QImage* image);
    void    drawInventory(QImage* image);
    int     getWeapon(int num);
    CRect   getBodyPartRect(CCharacter::EBodyParts part);
    CCharacter::SStat   getStat(int num, CCharacter::EStatsNames statName);
    void    displayStat(QImage* image, CVec2 pos, int x2, CCharacter::EStatsNames num);

    EMainState          mainState;
    EWeaponsAreaState   weaponsAreaState;
    int     currentChampion;
    int     currentWeapon;
    QTimer* weaponCoolDown[4];
    std::map<std::string, std::string>  textsDB;
    bool    isResurrecting;
    bool    isPressingEye;
};

extern CInterface   interface;

#endif // INTERFACE_H
