#include "tiles.h"
#include <QtXml/qdom.h>
#include <QFile>
#include "../common/sources/system/filecache.h"
#include "../common/sources/system/cgraphics2d.h"
#include "game.h"
#include <QDebug>

CTiles   tiles;

CTiles::CTiles()
{

}

//---------------------------------------------------------------------------------------------
void    CTiles::drawFloorOrnate(QImage* image, CVec2 tablePos, int ornate)
{
    COrnateData ornateData = ornatesDatas[ornate];
    bool    flip = false;

    if (tablePos.x >= (int)((WALL_TABLE_WIDTH + 1) / 2))
    {
        tablePos.x = WALL_TABLE_WIDTH - 1 - tablePos.x;
        flip = true;
    }

    COrnateElement  elem = ornateData.table[tablePos.y][tablePos.x];

    if (elem.image.isEmpty() == false)
    {
        QImage  ornateImage = fileCache.getImage(elem.image.toLocal8Bit().constData());
        CVec2   pos = elem.pos;

        if (flip == true)
            pos.x = MAINVIEW_WIDTH - pos.x - ornateImage.width();

        graph2D.drawImage(image, pos, ornateImage, 0, flip);
    }
}

//--------------------------------------------------------------------------------------------------------
void    CTiles::readFloorOrnatesDB()
{
    QDomDocument doc;

    QFile f("databases/floor_ornates.xml");
    f.open(QIODevice::ReadOnly);
    doc.setContent(&f);
    f.close();

    QDomElement root = doc.documentElement();
    QDomElement ornate = root.firstChild().toElement();

    while(!ornate.isNull())
    {
        if (ornate.tagName() == "floor_ornate")
        {
            COrnateData   newOrnate;

            newOrnate.name = ornate.attribute("name");

            QDomElement ornateInfo = ornate.firstChild().toElement();

            while(!ornateInfo.isNull())
            {
                QString tag = ornateInfo.tagName();

                if (tag.startsWith("image_") == true)
                {
                    std::string str = tag.toUtf8().constData();
                    int x = str.at(str.size() - 2) - '0';
                    int y = str.at(str.size() - 1) - '0';
                    COrnateElement* elem = &newOrnate.table[y][x];
                    elem->image = QString("gfx/3DView/floor_ornates/") + ornateInfo.text();
                }
                else if (tag.startsWith("pos_") == true)
                {
                    std::string str = tag.toUtf8().constData();
                    int x = str.at(str.size() - 2) - '0';
                    int y = str.at(str.size() - 1) - '0';
                    COrnateElement* elem = &newOrnate.table[y][x];
                    elem->pos.x = ornateInfo.attribute("x").toInt();
                    elem->pos.y = ornateInfo.attribute("y").toInt();
                }

                ornateInfo = ornateInfo.nextSibling().toElement();
            }
            ornatesDatas.push_back(newOrnate);
        }
        ornate = ornate.nextSibling().toElement();
    }
}

//---------------------------------------------------------------------------------------------
void    CTiles::readTilesDB()
{
    QDomDocument doc;

    QFile f("databases/tiles.xml");
    f.open(QIODevice::ReadOnly);
    doc.setContent(&f);
    f.close();

    QDomElement root = doc.documentElement();
    QDomElement tile = root.firstChild().toElement();

    while(!tile.isNull())
    {
        std::vector<CParamType> paramsTypesList;

        if (tile.tagName() == "tile")
        {
            QDomElement tileInfo = tile.firstChild().toElement();

            while(!tileInfo.isNull())
            {
                if (tileInfo.tagName() == "param")
                {
                    QString paramType = tileInfo.attribute("type");
                    CParamType  newParamType;

                    if (paramType == "ornate")
                    {
                        newParamType.mType = eParamOrnate;
                        newParamType.mName = tileInfo.text();
                        paramsTypesList.push_back(newParamType);
                    }
                    else if (paramType == "int")
                    {
                        newParamType.mType = eParamInt;
                        newParamType.mName = tileInfo.text();
                        paramsTypesList.push_back(newParamType);
                    }
                    else if (paramType == "bool")
                    {
                        newParamType.mType = eParamBool;
                        newParamType.mName = tileInfo.text();
                        paramsTypesList.push_back(newParamType);
                    }
                    else if (paramType == "door")
                    {
                        newParamType.mType = eParamDoor;
                        newParamType.mName = tileInfo.text();
                        paramsTypesList.push_back(newParamType);
                    }
                    else if (paramType == "door_ornate")
                    {
                        newParamType.mType = eParamDoorOrnate;
                        newParamType.mName = tileInfo.text();
                        paramsTypesList.push_back(newParamType);
                    }
                }

                tileInfo = tileInfo.nextSibling().toElement();
            }
            map.mTilesParams.push_back(paramsTypesList);
        }
        tile = tile.nextSibling().toElement();
    }
}
